/******************************************************************************
*     WMSound: Window Maker Sound Server                                      *
*     Copyright (C) 1998, 1999                                                *
*         Anthony Quinn   <southgat@frontiernet.net>                          *
*         Pascal Hofstee  <daeron@shadowmere.student.utwente.nl>              *
*                                                                             *
*     This program is free software: you can redistribute it and/or modify    *
*     it under the terms of the GNU General Public License as published by    *
*     the Free software Foundattion; either version 2 of the License, or      *
*     (at your option) any later version.                                     *
*                                                                             *
*     This program is distributed in the hope that it will be usefull,        *
*     but WITHOUT ANY WARRANTY; without even the implied warranty of          *
*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            *
*     GNU General Public License for more details.                            *
*                                                                             *
*     You should have received a copy of the GNU General Public License       *
*     along with this program; if not, write to the Free Software             *
*     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.               *
******************************************************************************/


#include "wmsnd.h"

proplist_t
wmsndGetObjectForKey(char *defaultName)
{
	proplist_t object = NULL;
	proplist_t key = PLMakeString(defaultName);
	
	object = PLGetDictionaryEntry(WMSoundDB, key);
	
	/*
	 * Don't use a Global/Local Approach in this application
	 * if (!object)
	 * object = PLGetDictionaryEntry(GlobalDB, key);
	 */
	
	PLRelease(key);
	
	return object;
}


void
wmsndSetObjectForKey(proplist_t object, char *defaultName)
{
	proplist_t key = PLMakeString(defaultName);
	
	PLInsertDictionaryEntry(WMSoundDB, key, object);
	
	PLRelease(key);
}


void
wmsndRemoveObjectForKey(char *defaultName)
{
	proplist_t key = PLMakeString(defaultName);
	
	PLRemoveDictionaryEntry(WMSoundDB, key);
	
	PLRelease(key);
}


char*
wmsndGetStringForKey(char *defaultName)
{
	proplist_t val;
	
	val = wmsndGetObjectForKey(defaultName);
	
	if (!val)
		return NULL;
	
	if (!PLIsString(val))
		return NULL;
	
	return PLGetString(val);
}



proplist_t
wmsndGetArrayForKey(char *defaultName)
{
	proplist_t val;
	
	val = wmsndGetObjectForKey(defaultName);
	
	if (!val)
		return NULL;
	
	if (!PLIsArray(val))
		return NULL;
	
	return val;
}


proplist_t
wmsndGetDictionaryForKey(char *defaultName)
{
	proplist_t val;
	
	val = wmsndGetObjectForKey(defaultName);
	
	if (!val)
		return NULL;
	
	if (!PLIsDictionary(val))
		return NULL;
	
	return val;
}


void
wmsndSetStringForKey(char *value, char *defaultName)
{
	proplist_t object;
	
	object = PLMakeString(value);
	
	wmsndSetObjectForKey(object, defaultName);
	PLRelease(object);
}

